@@ -1,6 +1,6 @@
 /* Definitions of target machine for GNU compiler, for HP PA-RISC
    Copyright (C) 1995, 1996, 1997, 2000, 2001, 2002, 2003, 2004,
-   2007 Free Software Foundation, Inc.
+   2007, 2008 Free Software Foundation, Inc.
    Contributed by Tim Moore (moore@defmacro.cs.utah.edu)
 
 This file is part of GCC.
@@ -84,7 +84,8 @@ along with GCC; see the file COPYING3.  If not see
 #undef LINK_SPEC
 #if ((TARGET_DEFAULT | TARGET_CPU_DEFAULT) & MASK_PA_11)
 #define LINK_SPEC \
-  "%{!mpa-risc-1-0:%{!march=1.0:%{static:-L/lib/pa1.1 -L/usr/lib/pa1.1 }}}\
+  "%<fwhole-program\
+   %{!mpa-risc-1-0:%{!march=1.0:%{static:-L/lib/pa1.1 -L/usr/lib/pa1.1 }}}\
    %{!shared:%{p:-L/lib/libp %{!static:\
      %nWarning: consider linking with `-static' as system libraries with\n\
      %n  profiling support are only provided in archive format}}}\
@@ -95,7 +96,8 @@ along with GCC; see the file COPYING3.  If not see
    %{static:-a archive} %{shared:-b}"
 #else
 #define LINK_SPEC \
-  "%{!shared:%{p:-L/lib/libp %{!static:\
+  "%<fwhole-program\
+   %{!shared:%{p:-L/lib/libp %{!static:\
      %nWarning: consider linking with `-static' as system libraries with\n\
      %n  profiling support are only provided in archive format}}}\
    %{!shared:%{pg:-L/lib/libp %{!static:\
