@@ -2,6 +2,8 @@
 #include <objc/objc.h>
 #include <objc/objc-api.h>
 
+#include "next_mapping.h"
+
 /* Tests creating a root class and a subclass with an ivar and
    accessor methods */
 
@@ -12,6 +14,9 @@
 @end
 
 @implementation RootClass
+#ifdef __NEXT_RUNTIME__                                   
++ initialize { return self; }
+#endif
 @end
 
 @interface SubClass : RootClass
