@@ -6,7 +6,7 @@
    PR:		none.
    Originator:	<andreast@gcc.gnu.org> 20031026	 */
 
-/* { dg-do run { xfail mips64*-*-* } } */
+/* { dg-do run } */
 #include "ffitest.h"
 
 static void
@@ -74,11 +74,11 @@ int main (void)
   for (i = 0; i < 10; i++) {
     cl_arg_types[i] = &ffi_type_uint64;
   }
-  cl_arg_types[10] = &ffi_type_uint;
+  cl_arg_types[10] = &ffi_type_sint;
   for (i = 11; i < 15; i++) {
     cl_arg_types[i] = &ffi_type_uint64;
   }
-  cl_arg_types[15] = &ffi_type_uint;
+  cl_arg_types[15] = &ffi_type_sint;
   cl_arg_types[16] = NULL;
 
   /* Initialize the cif */
