@@ -1,9 +1,9 @@
-// Copyright (C) 2003 Free Software Foundation
+// Copyright (C) 2003, 2009 Free Software Foundation
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the
-// Free Software Foundation; either version 2, or (at your option)
+// Free Software Foundation; either version 3, or (at your option)
 // any later version.
 
 // This library is distributed in the hope that it will be useful,
@@ -12,9 +12,8 @@
 // GNU General Public License for more details.
 
 // You should have received a copy of the GNU General Public License along
-// with this library; see the file COPYING.  If not, write to the Free
-// Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
-// USA.
+// with this library; see the file COPYING3.  If not see
+// <http://www.gnu.org/licenses/>.
 
 // 22.2.5.3.1 time_put members
 
@@ -24,37 +23,41 @@
 #include <cstring>
 #include <testsuite_hooks.h>
 
+class TP : public std::time_put<wchar_t>
+{
+public:
+  mutable std::wstring fill_chars;
+
+protected:
+  iter_type do_put(iter_type s, std::ios_base&, char_type fill,
+		   const std::tm*, char, char) const
+  {
+    fill_chars.push_back(fill);
+    return s;
+  }
+};
+
 // libstdc++/12439
-// time_put::put writes narrowed characters to output iterator
-void test02()
+// time_put::put doesn't pass fill character to do_put
+void test01()
 {
   using namespace std;
   bool test __attribute__((unused)) = true;
-
-  typedef time_put<wchar_t> tp_type;
   
-  const wchar_t fmt[] = {
-    0xa0, 0x103, 0xfc, 0xb3, 0xa0c3,
-    L'%', L'c'
-  };
-  
-  const size_t len = sizeof(fmt) / sizeof(fmt[0]);
-  const size_t cmplen = wcschr(fmt, L'%') - fmt;
-  
-  locale loc;
-  const tp_type& tp = use_facet<tp_type>(loc);
-  time_t tt = time(NULL);
   wostringstream stream;
+  time_t tt = time(NULL);
   
-  tp.put(tp_type::iter_type(stream), stream, stream.fill(),
-	 localtime(&tt), fmt, fmt + len);
-  wstring str = stream.str();
-  VERIFY( str.length() >= cmplen );
-  VERIFY( !wmemcmp(str.data(), fmt, cmplen) );
+  const wchar_t* fmt = L"%c";
+  
+  TP tp;
+  tp.put(TP::iter_type(stream), stream, L'W', localtime(&tt),
+	 fmt, fmt + wcslen(fmt));
+  VERIFY( !tp.fill_chars.empty() );
+  VERIFY( tp.fill_chars[tp.fill_chars.length() - 1] == L'W' );
 }
 
 int main()
 {
-  test02();
+  test01();
   return 0;
 }
