@@ -1,11 +1,11 @@
-// 2003-05-01 Petur Runolfsson  <peturr02@ru.is>
+// 2003-04-30  Petur Runolfsson <peturr02@ru.is>
 
-// Copyright (C) 2000, 2001, 2002, 2003 Free Software Foundation, Inc.
+// Copyright (C) 2003, 2009 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the
-// Free Software Foundation; either version 2, or (at your option)
+// Free Software Foundation; either version 3, or (at your option)
 // any later version.
 
 // This library is distributed in the hope that it will be useful,
@@ -14,25 +14,34 @@
 // GNU General Public License for more details.
 
 // You should have received a copy of the GNU General Public License along
-// with this library; see the file COPYING.  If not, write to the Free
-// Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
-// USA.
+// with this library; see the file COPYING3.  If not see
+// <http://www.gnu.org/licenses/>.
 
+#include <testsuite_hooks.h>
+#include <cstdio>
 #include <iostream>
+#include <cwchar>
 
-// libstdc++/6648
-// Interactive tests: each one (run alone) must terminate upon a single '\n'.
-void test08()
+void test02()
 {
+  using namespace std;
+
   bool test __attribute__((unused)) = true;
-  wchar_t buff[2048];
-  std::wcout << "Enter name: ";
-  std::wcin.getline(buff, 2048);
+
+  wint_t c1 = fgetwc(stdin);
+  wint_t c2 = wcin.rdbuf()->sputbackc(c1);
+  VERIFY( c2 == c1 );
+  
+  wint_t c3 = fgetwc(stdin);
+  VERIFY( c3 == c1 );
+  ungetwc(c3, stdin);
+  
+  wint_t c4 = wcin.rdbuf()->sgetc();
+  VERIFY( c4 == c3 );
 }
 
-int 
-main()
+int main()
 {
-  test08();
+  test02();
   return 0;
 }
